/*
 * WickedShellPreferencePage.java
 * 
 * Copyright 2005-2006 Stefan Reichert.
 * All Rights Reserved.
 * 
 * This software is the proprietary information of Stefan Reichert.
 * Use is subject to license terms.
 * 
 */
package net.sf.wickedshell.ui.preferences;

import java.util.List;

import net.sf.wickedshell.ShellID;
import net.sf.wickedshell.ShellPlugin;
import net.sf.wickedshell.domain.DomainPlugin;
import net.sf.wickedshell.domain.completion.ICompletion;
import net.sf.wickedshell.domain.configuration.IShellDescriptorProperties;
import net.sf.wickedshell.domain.style.IColorDescriptor;
import net.sf.wickedshell.domain.style.IFontDescriptor;
import net.sf.wickedshell.facade.descriptor.IShellDescriptor;
import net.sf.wickedshell.preferences.PreferenceHelper;
import net.sf.wickedshell.provider.ShellDescriptorLabelProvider;
import net.sf.wickedshell.util.ResourceManager;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * @author Stefan Reichert
 * @since 24.03.2006
 */
public class WickedShellPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	private Button buttonBrowseCustomStartupDirectory;
	private Text textCustomStartupDirectory;
	private Button buttonUseWorkspaceAsStartupDirectory;
	private Button buttonBrowseShellRootDirectory;
	private Button buttonShowCascadingCompletions;
	private Button buttonChooseFont;
	private Label labelShellStyle;
	private Text textShellRootDirectory;
	private ComboViewer comboViewerShellDescriptor;
	private TableViewer tableViewerStaticCompletions;
	private Button buttonOpenShellErrorDialog;
	private Button buttonSystemPathCompletions;
	private Button buttonShowActiveFolderCompletions;
	private Button buttonShowStaticCompletions;
	private Button buttonSystemEnviromentalValueCompletions;

	private IFontDescriptor shellFontDescriptor;
	private RGB shellForegroundRGB;
	private RGB shellBackgroundRGB;

	/**
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public Control createContents(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayoutContainer = new GridLayout();
		gridLayoutContainer.marginWidth = 2;
		gridLayoutContainer.horizontalSpacing = 2;
		gridLayoutContainer.marginHeight = 2;
		container.setLayout(gridLayoutContainer);

		final Group groupShellSettings = new Group(container, SWT.NONE);
		final GridLayout gridLayoutShell = new GridLayout();
		gridLayoutShell.numColumns = 3;
		groupShellSettings.setLayout(gridLayoutShell);
		groupShellSettings.setText("Shell settings");
		groupShellSettings.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));

		final Label labelShell = new Label(groupShellSettings, SWT.NONE);
		labelShell.setLayoutData(new GridData());
		labelShell.setText("Default Shell:");

		comboViewerShellDescriptor = new ComboViewer(groupShellSettings, SWT.READ_ONLY);
		final Combo comboShellDescriptor = comboViewerShellDescriptor.getCombo();
		comboShellDescriptor
				.setToolTipText("Please select the shell to be used. Ensure the selected shell to be installed and the executable is referenced in the path.");
		comboShellDescriptor.select(0);
		comboViewerShellDescriptor.setContentProvider(new ArrayContentProvider());
		comboViewerShellDescriptor.setLabelProvider(new ShellDescriptorLabelProvider());
		comboViewerShellDescriptor.addFilter(new ViewerFilter() {
			/**
			 * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer,
			 *      java.lang.Object, java.lang.Object)
			 */
			public boolean select(Viewer viewer, Object parentElement, Object element) {
				IShellDescriptor descriptor = (IShellDescriptor) element;
				return descriptor.isCurrentOSSupported();
			}
		});

		final GridData gridData = new GridData(GridData.FILL, GridData.CENTER, true, false, 2, 1);
		gridData.horizontalIndent = 25;
		comboShellDescriptor.setLayoutData(gridData);

		final Label labelShellRootDirectory = new Label(groupShellSettings, SWT.NONE);
		labelShellRootDirectory.setText("Shell root directory:");

		textShellRootDirectory = new Text(groupShellSettings, SWT.BORDER);
		textShellRootDirectory
				.setToolTipText("If you use an installed shell with a specific root directory, please enter this root directory here.");
		final GridData gridDataTextShellRootDirectory = new GridData(GridData.FILL, GridData.CENTER,
				true, false);
		gridDataTextShellRootDirectory.horizontalIndent = 25;
		textShellRootDirectory.setLayoutData(gridDataTextShellRootDirectory);

		buttonBrowseShellRootDirectory = new Button(groupShellSettings, SWT.NONE);
		buttonBrowseShellRootDirectory.setText("Browse...");
		buttonBrowseShellRootDirectory.addSelectionListener(new SelectionAdapter() {
			/**
			 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetSelected(SelectionEvent selectionEvent) {
				DirectoryDialog shellRootDirectorySelectionDialog = new DirectoryDialog(getShell());
				String shellRootDirectory = shellRootDirectorySelectionDialog.open();
				if (shellRootDirectory != null) {
					textShellRootDirectory.setText(shellRootDirectory);
				}
			}
		});

		comboViewerShellDescriptor.addSelectionChangedListener(new ISelectionChangedListener() {
			/**
			 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
			 */
			public void selectionChanged(SelectionChangedEvent event) {
				if (!event.getSelection().isEmpty()) {
					IStructuredSelection selection = (IStructuredSelection) event.getSelection();
					IShellDescriptor descriptor = (IShellDescriptor) selection.getFirstElement();
					if (descriptor.hasCustomRoot()) {
						textShellRootDirectory.setEnabled(true);
						buttonBrowseShellRootDirectory.setEnabled(true);
						IShellDescriptorProperties shellDescriptorProperties = DomainPlugin.getDefault()
								.getShellDescriptorProperties(descriptor.getId());
						textShellRootDirectory.setText(shellDescriptorProperties.getRootDirectory());

					}
					else {
						textShellRootDirectory.setText(new String());
						textShellRootDirectory.setEnabled(false);
						buttonBrowseShellRootDirectory.setEnabled(false);
					}
				}
				else {
					textShellRootDirectory.setText(new String());
					textShellRootDirectory.setEnabled(false);
					buttonBrowseShellRootDirectory.setEnabled(false);
				}
			}
		});

		// Set the viewer's inputs
		try {
			comboViewerShellDescriptor.setInput(IShellDescriptor.Manager.getKnownShellDescriptors());
			IShellDescriptor shellDescriptor = PreferenceHelper.getActiveShellDescriptor();
			IShellDescriptorProperties shellDescriptorProperties = DomainPlugin.getDefault()
					.getShellDescriptorProperties(shellDescriptor.getId());
			comboViewerShellDescriptor.setSelection(new StructuredSelection(shellDescriptor));
			textShellRootDirectory.setText(shellDescriptorProperties.getRootDirectory());

		}
		catch (Exception exception) {
			exception.printStackTrace();
		}

		final Group groupGeneralSettings = new Group(container, SWT.NONE);
		groupGeneralSettings.setText("General settings");
		groupGeneralSettings.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
		final GridLayout gridLayout = new GridLayout();
		groupGeneralSettings.setLayout(gridLayout);

		buttonOpenShellErrorDialog = new Button(groupGeneralSettings, SWT.CHECK);
		buttonOpenShellErrorDialog.setLayoutData(new GridData());
		buttonOpenShellErrorDialog.setText("Open dialog on error in Shell");
		buttonOpenShellErrorDialog.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.OPEN_SHELL_ERROR_DIALOG_STRING_KEY));

		buttonShowStaticCompletions = new Button(groupGeneralSettings, SWT.CHECK);
		buttonShowStaticCompletions.setLayoutData(new GridData());
		buttonShowStaticCompletions.setText("Suggest static completions in Shell");
		buttonShowStaticCompletions.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.SHOW_CUSTOM_COMMANDS_STRING_KEY));

		buttonShowActiveFolderCompletions = new Button(groupGeneralSettings, SWT.CHECK);
		buttonShowActiveFolderCompletions.setLayoutData(new GridData());
		buttonShowActiveFolderCompletions.setText("Suggest active folder completions in Shell");
		buttonShowActiveFolderCompletions.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.SHOW_ACTIVE_FOLDER_COMMANDS_STRING_KEY));

		buttonShowCascadingCompletions = new Button(groupGeneralSettings, SWT.CHECK);
		buttonShowCascadingCompletions.setLayoutData(new GridData());
		buttonShowCascadingCompletions.setText("Suggest cascading completions in Shell");
		buttonShowCascadingCompletions.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.SHOW_CASCADING_COMPLETIONS_STRING_KEY));

		buttonSystemPathCompletions = new Button(groupGeneralSettings, SWT.CHECK);
		buttonSystemPathCompletions.setLayoutData(new GridData());
		buttonSystemPathCompletions.setText("Suggest system path completions in Shell");
		buttonSystemPathCompletions.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.SHOW_SYSTEM_PATH_COMMANDS_STRING_KEY));

		buttonSystemEnviromentalValueCompletions = new Button(groupGeneralSettings, SWT.CHECK);
		buttonSystemEnviromentalValueCompletions.setLayoutData(new GridData());
		buttonSystemEnviromentalValueCompletions
				.setText("Suggest enviromental value completions in Shell");
		buttonSystemEnviromentalValueCompletions.setSelection(ShellPlugin.getDefault()
				.getPreferenceStore().getBoolean(ShellID.SHOW_SYSTEM_ENVIROMENTAL_VALUES_STRING_KEY));

		final Group groupStartupSettings = new Group(container, SWT.NONE);
		groupStartupSettings.setText("Startup settings (if supported by selected Shell)");
		groupStartupSettings.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		final GridLayout gridLayoutGroupStartupSettings = new GridLayout();
		gridLayoutGroupStartupSettings.numColumns = 3;
		groupStartupSettings.setLayout(gridLayoutGroupStartupSettings);

		buttonUseWorkspaceAsStartupDirectory = new Button(groupStartupSettings, SWT.CHECK);
		buttonUseWorkspaceAsStartupDirectory.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 3, 1));
		buttonUseWorkspaceAsStartupDirectory.setText("Use workspace as startup directory");
		buttonUseWorkspaceAsStartupDirectory
				.setSelection(ShellPlugin.getDefault().getPreferenceStore().getBoolean(
						ShellID.USE_WORKSPACE_AS_STARTUP_DIRECTORY_STRING_KEY));
		buttonUseWorkspaceAsStartupDirectory.addSelectionListener(new SelectionAdapter() {
			/**
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetSelected(SelectionEvent event) {
				if (buttonUseWorkspaceAsStartupDirectory.getSelection()) {
					textCustomStartupDirectory.setText(new String());
				}
				else {
					textCustomStartupDirectory.setText(ShellPlugin.getDefault().getPreferenceStore()
							.getString(ShellID.CUSTOM_STARTUP_DIRECTORY_STRING_KEY));
				}
				textCustomStartupDirectory.setEnabled(!buttonUseWorkspaceAsStartupDirectory
						.getSelection());
				buttonBrowseCustomStartupDirectory.setEnabled(!buttonUseWorkspaceAsStartupDirectory
						.getSelection());
			}
		});

		final Label customStartupDirectoryLabel = new Label(groupStartupSettings, SWT.NONE);
		customStartupDirectoryLabel.setText("Custom startup directory:");

		textCustomStartupDirectory = new Text(groupStartupSettings, SWT.BORDER);
		textCustomStartupDirectory.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		textCustomStartupDirectory.setText(ShellPlugin.getDefault().getPreferenceStore().getString(
				ShellID.CUSTOM_STARTUP_DIRECTORY_STRING_KEY));
		textCustomStartupDirectory.setEnabled(!buttonUseWorkspaceAsStartupDirectory.getSelection());

		buttonBrowseCustomStartupDirectory = new Button(groupStartupSettings, SWT.NONE);
		buttonBrowseCustomStartupDirectory.setText("Browse...");
		buttonBrowseCustomStartupDirectory.setEnabled(!buttonUseWorkspaceAsStartupDirectory
				.getSelection());
		buttonBrowseCustomStartupDirectory.addSelectionListener(new SelectionAdapter() {
			/**
			 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetSelected(SelectionEvent selectionEvent) {
				DirectoryDialog shellCustomStartupDirectorySelectionDialog = new DirectoryDialog(
						getShell());
				String customStartupDirectory = shellCustomStartupDirectorySelectionDialog.open();
				if (customStartupDirectory != null) {
					textCustomStartupDirectory.setText(customStartupDirectory);
				}
			}
		});

		final Group groupStyleSettings = new Group(container, SWT.NONE);
		groupStyleSettings.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
		groupStyleSettings.setText("Style settings");
		final GridLayout gridLayoutGroupStyleSettings = new GridLayout();
		gridLayoutGroupStyleSettings.numColumns = 3;
		groupStyleSettings.setLayout(gridLayoutGroupStyleSettings);

		labelShellStyle = new Label(groupStyleSettings, SWT.BORDER);
		final GridData gridDataLabelShellFontName = new GridData(GridData.FILL, GridData.CENTER,
				true, true, 3, 1);
		gridDataLabelShellFontName.heightHint = 40;
		labelShellStyle.setLayoutData(gridDataLabelShellFontName);
		handleFontDescriptorSelection(DomainPlugin.getDefault().getFontDescriptor());

		buttonChooseFont = new Button(groupStyleSettings, SWT.NONE);
		buttonChooseFont.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));
		buttonChooseFont.setText("Choose Font...");
		buttonChooseFont.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				FontDialog fontDialog = new FontDialog(getShell());
				fontDialog.setFontList(ResourceManager.getFont(shellFontDescriptor.getName(),
						shellFontDescriptor.getHeight(), shellFontDescriptor.getStyle()).getFontData());
				fontDialog.setText("Select Shell Font");
				FontData fontData = fontDialog.open();
				if (fontData != null) {
					handleFontDescriptorSelection(IFontDescriptor.Factory.newInstance(
							fontData.getName(), fontData.getHeight(), fontData.getStyle()));
				}
			}
		});
		IColorDescriptor foregroundColorDescriptor = DomainPlugin.getDefault().getForegroundColor();
		handleForegroundRGBSelection(new RGB(foregroundColorDescriptor.getRed(),
				foregroundColorDescriptor.getGreen(), foregroundColorDescriptor.getBlue()));
		final Button buttonShellForegroundColor = new Button(groupStyleSettings, SWT.NONE);
		buttonShellForegroundColor.setText("Choose Foreground Color...");
		buttonShellForegroundColor.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING,
				true, false));
		buttonShellForegroundColor.addSelectionListener(new SelectionAdapter() {
			/**
			 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetSelected(SelectionEvent event) {
				ColorDialog colorDialog = new ColorDialog(getShell());
				colorDialog.setRGB(shellBackgroundRGB);
				RGB selectedForegroundRGB = colorDialog.open();
				if (selectedForegroundRGB != null) {
					handleForegroundRGBSelection(selectedForegroundRGB);
				}
			}
		});
		IColorDescriptor backgroundColorDescriptor = DomainPlugin.getDefault().getBackgroundColor();
		handleBackgroundRGBSelection(new RGB(backgroundColorDescriptor.getRed(),
				backgroundColorDescriptor.getGreen(), backgroundColorDescriptor.getBlue()));

		final Button buttonShellBackgroundColor = new Button(groupStyleSettings, SWT.NONE);
		buttonShellBackgroundColor.setSelection(true);
		buttonShellBackgroundColor.setText("Choose Background Color...");
		final GridData gridDataButtonShellBackgroundColor = new GridData(GridData.FILL,
				GridData.BEGINNING, true, false);
		buttonShellBackgroundColor.setLayoutData(gridDataButtonShellBackgroundColor);
		buttonShellBackgroundColor.addSelectionListener(new SelectionAdapter() {
			/**
			 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetSelected(SelectionEvent event) {
				ColorDialog colorDialog = new ColorDialog(getShell());
				colorDialog.setRGB(shellBackgroundRGB);
				RGB selectedBackgroundRGB = colorDialog.open();
				if (selectedBackgroundRGB != null) {
					handleBackgroundRGBSelection(selectedBackgroundRGB);
				}
			}
		});

		final Group groupStaticCompletions = new Group(container, SWT.NONE);
		groupStaticCompletions.setText("Static completions");
		groupStaticCompletions.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		final GridLayout gridLayoutGroupStaticCompletions = new GridLayout();
		gridLayoutGroupStaticCompletions.numColumns = 2;
		groupStaticCompletions.setLayout(gridLayoutGroupStaticCompletions);

		tableViewerStaticCompletions = new TableViewer(groupStaticCompletions, SWT.MULTI
				| SWT.FULL_SELECTION | SWT.BORDER);
		tableViewerStaticCompletions.setContentProvider(new ArrayContentProvider());
		tableViewerStaticCompletions.setLabelProvider(new CompletionTableLabelProvider());
		tableViewerStaticCompletions.setSorter(new ViewerSorter() {
			/**
			 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer,
			 *      java.lang.Object, java.lang.Object)
			 */
			public int compare(Viewer viewer, Object objectOne, Object objectTwo) {
				ICompletion completionOne = (ICompletion) objectOne;
				ICompletion completionTwo = (ICompletion) objectTwo;
				return super.compare(viewer, completionOne.getContent(), completionTwo.getContent());
			}
		});

		final Table tableStaticCompletions = tableViewerStaticCompletions.getTable();
		tableStaticCompletions.setLinesVisible(true);
		tableStaticCompletions.setHeaderVisible(true);
		final GridData gridData_1 = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 3);
		gridData_1.heightHint = 150;
		tableStaticCompletions.setLayoutData(gridData_1);

		final TableColumn completionTableColumn = new TableColumn(tableStaticCompletions, SWT.NONE);
		completionTableColumn.setWidth(150);
		completionTableColumn.setText("Completion");

		final TableColumn descriptionTableColumn = new TableColumn(tableStaticCompletions, SWT.NONE);
		descriptionTableColumn.setWidth(250);
		descriptionTableColumn.setText("Description");

		final Button buttonAdd = new Button(groupStaticCompletions, SWT.NONE);
		final GridData gridDataButtonAdd = new GridData(GridData.FILL, GridData.CENTER, false, false);
		gridDataButtonAdd.horizontalIndent = 10;
		buttonAdd.setLayoutData(gridDataButtonAdd);
		buttonAdd.setText("Add completion");
		buttonAdd.addSelectionListener(new SelectionAdapter() {
			/**
			 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetSelected(SelectionEvent event) {
				AddStaticCompletionDialog dialog = new AddStaticCompletionDialog(getShell());
				if (dialog.open() == IDialogConstants.OK_ID) {
					StringBuffer descriptionBuffer = new StringBuffer();
					descriptionBuffer.append(dialog.getCompletion());
					descriptionBuffer.append(" - ");
					descriptionBuffer.append(dialog.getComment());
					descriptionBuffer.append(" (Static)");

					List staticCompletions = DomainPlugin.getDefault().getStaticCompletions();
					staticCompletions.add(ICompletion.Factory.newInstance(dialog.getComment(),
							descriptionBuffer.toString(), new String()));
					tableViewerStaticCompletions.setInput(staticCompletions);
				}
			};
		});

		final Button buttonRemove = new Button(groupStaticCompletions, SWT.NONE);
		final GridData gridDataButtonRemove = new GridData(GridData.FILL, GridData.CENTER, false,
				false);
		gridDataButtonRemove.horizontalIndent = 10;
		buttonRemove.setLayoutData(gridDataButtonRemove);
		buttonRemove.setText("Remove completion");
		buttonRemove.addSelectionListener(new SelectionAdapter() {
			/**
			 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			public void widgetSelected(SelectionEvent event) {
				if (!tableViewerStaticCompletions.getSelection().isEmpty()) {
					IStructuredSelection selection = (IStructuredSelection) tableViewerStaticCompletions
							.getSelection();
					List staticCompletions = DomainPlugin.getDefault().getStaticCompletions();
					staticCompletions.removeAll(selection.toList());
					tableViewerStaticCompletions.setInput(staticCompletions);
				}
			};
		});
		// Set the viewer's input
		tableViewerStaticCompletions.setInput(DomainPlugin.getDefault().getStaticCompletions());
		return container;
	}

	/**
	 * Handles the selection of a <code>Font</code>.
	 * 
	 * @param selectedFont
	 *        The selected <code>Font</code>
	 */
	private void handleFontDescriptorSelection(IFontDescriptor selectedFontDescriptor) {
		shellFontDescriptor = selectedFontDescriptor;
		StringBuffer buffer = new StringBuffer();
		buffer.append("Style of Wicked Shell ");
		buffer.append(ShellID.VERSION);
		buffer.append(Text.DELIMITER);
		buffer.append("Chosen Font: ");
		buffer.append(shellFontDescriptor.getName());
		buffer.append(" [");
		buffer.append(shellFontDescriptor.getHeight());
		buffer.append("]");
		labelShellStyle.setText(buffer.toString());
		labelShellStyle.setFont(ResourceManager.getFont(shellFontDescriptor.getName(),
				shellFontDescriptor.getHeight(), shellFontDescriptor.getStyle()));
	}

	/**
	 * Handles the selection of a foreground <code>RGB</code>.
	 * 
	 * @param selectedForegroundRGB
	 *        The selected foreground <code>RGB</code>
	 */
	private void handleForegroundRGBSelection(RGB selectedForegroundRGB) {
		shellForegroundRGB = selectedForegroundRGB;
		labelShellStyle.setForeground(ResourceManager.getColor(selectedForegroundRGB));
	}

	/**
	 * Handles the selection of a background <code>RGB</code>.
	 * 
	 * @param selectedBackgroundRGB
	 *        The selected background <code>RGB</code>
	 */
	private void handleBackgroundRGBSelection(RGB selectedBackgroundRGB) {
		shellBackgroundRGB = selectedBackgroundRGB;
		labelShellStyle.setBackground(ResourceManager.getColor(selectedBackgroundRGB));
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	/**
	 * @see org.eclipse.jface.preference.PreferencePage#doGetPreferenceStore()
	 */
	protected IPreferenceStore doGetPreferenceStore() {
		return ShellPlugin.getDefault().getPreferenceStore();
	}

	/**
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	public boolean performOk() {
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.OPEN_SHELL_ERROR_DIALOG_STRING_KEY, buttonOpenShellErrorDialog.getSelection());
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.SHOW_CUSTOM_COMMANDS_STRING_KEY, buttonShowStaticCompletions.getSelection());
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.SHOW_ACTIVE_FOLDER_COMMANDS_STRING_KEY,
				buttonShowActiveFolderCompletions.getSelection());
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.SHOW_CASCADING_COMPLETIONS_STRING_KEY,
				buttonShowCascadingCompletions.getSelection());
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.SHOW_SYSTEM_PATH_COMMANDS_STRING_KEY,
				buttonSystemPathCompletions.getSelection());
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.SHOW_SYSTEM_ENVIROMENTAL_VALUES_STRING_KEY,
				buttonSystemEnviromentalValueCompletions.getSelection());
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.USE_WORKSPACE_AS_STARTUP_DIRECTORY_STRING_KEY,
				buttonUseWorkspaceAsStartupDirectory.getSelection());
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.CUSTOM_STARTUP_DIRECTORY_STRING_KEY, textCustomStartupDirectory.getText());

		IStructuredSelection structuredSelection = (IStructuredSelection) comboViewerShellDescriptor
				.getSelection();
		IShellDescriptor shellDescriptor = (IShellDescriptor) structuredSelection.getFirstElement();
		ShellPlugin.getDefault().getPreferenceStore().setValue(
				ShellID.SHELL_DESCRIPTOR_ID_STRING_KEY, shellDescriptor.getId());
		IShellDescriptorProperties shellDescriptorProperties = DomainPlugin.getDefault()
				.getShellDescriptorProperties(shellDescriptor.getId());
		shellDescriptorProperties.setRootDirectory(textShellRootDirectory.getText());

		DomainPlugin.getDefault().setFontDescriptor(shellFontDescriptor);
		DomainPlugin.getDefault().setForegroundColor(
				IColorDescriptor.Factory.newInstance(shellForegroundRGB.red, shellForegroundRGB.green,
						shellForegroundRGB.blue));
		DomainPlugin.getDefault().setBackgroundColor(
				IColorDescriptor.Factory.newInstance(shellBackgroundRGB.red, shellBackgroundRGB.green,
						shellBackgroundRGB.blue));
		ShellPlugin.getDefault().getPreferenceStore().firePropertyChangeEvent(
				ShellID.SHELL_STYLE_STRING_KEY, null, null);
		return super.performOk();
	}

	/**
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.OPEN_SHELL_ERROR_DIALOG_STRING_KEY);
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.SHOW_CUSTOM_COMMANDS_STRING_KEY);
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.SHOW_ACTIVE_FOLDER_COMMANDS_STRING_KEY);
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.SHOW_CASCADING_COMPLETIONS_STRING_KEY);
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.SHOW_SYSTEM_PATH_COMMANDS_STRING_KEY);
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.SHOW_SYSTEM_ENVIROMENTAL_VALUES_STRING_KEY);
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.SHELL_DESCRIPTOR_ID_STRING_KEY);
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.USE_WORKSPACE_AS_STARTUP_DIRECTORY_STRING_KEY);
		ShellPlugin.getDefault().getPreferenceStore().setToDefault(
				ShellID.CUSTOM_STARTUP_DIRECTORY_STRING_KEY);

		buttonOpenShellErrorDialog.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.OPEN_SHELL_ERROR_DIALOG_STRING_KEY));
		buttonShowStaticCompletions.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.SHOW_CUSTOM_COMMANDS_STRING_KEY));
		buttonShowActiveFolderCompletions.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.SHOW_ACTIVE_FOLDER_COMMANDS_STRING_KEY));
		buttonShowCascadingCompletions.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.SHOW_CASCADING_COMPLETIONS_STRING_KEY));
		buttonSystemPathCompletions.setSelection(ShellPlugin.getDefault().getPreferenceStore()
				.getBoolean(ShellID.SHOW_SYSTEM_PATH_COMMANDS_STRING_KEY));
		buttonSystemEnviromentalValueCompletions.setSelection(ShellPlugin.getDefault()
				.getPreferenceStore().getBoolean(ShellID.SHOW_SYSTEM_ENVIROMENTAL_VALUES_STRING_KEY));
		buttonUseWorkspaceAsStartupDirectory
				.setSelection(ShellPlugin.getDefault().getPreferenceStore().getBoolean(
						ShellID.USE_WORKSPACE_AS_STARTUP_DIRECTORY_STRING_KEY));
		textCustomStartupDirectory.setText(ShellPlugin.getDefault().getPreferenceStore().getString(
				ShellID.CUSTOM_STARTUP_DIRECTORY_STRING_KEY));

		IShellDescriptor shellDescriptor = PreferenceHelper.getActiveShellDescriptor();
		comboViewerShellDescriptor.setSelection(new StructuredSelection(shellDescriptor));

		DomainPlugin.getDefault().resetShellDescriptorProperties();
		IShellDescriptorProperties shellDescriptorProperties = DomainPlugin.getDefault()
				.getShellDescriptorProperties(shellDescriptor.getId());
		textShellRootDirectory.setText(shellDescriptorProperties.getRootDirectory());

		DomainPlugin.getDefault().resetCompletions();
		tableViewerStaticCompletions.setInput(DomainPlugin.getDefault().getStaticCompletions());

		DomainPlugin.getDefault().resetShellStyle();
		handleFontDescriptorSelection(DomainPlugin.getDefault().getFontDescriptor());

		IColorDescriptor foregroundColorDescriptor = DomainPlugin.getDefault().getForegroundColor();
		handleForegroundRGBSelection(new RGB(foregroundColorDescriptor.getRed(),
				foregroundColorDescriptor.getGreen(), foregroundColorDescriptor.getBlue()));
		IColorDescriptor defaultBackgroundColorDescriptor = DomainPlugin.getDefault()
				.getBackgroundColor();
		handleBackgroundRGBSelection(new RGB(defaultBackgroundColorDescriptor.getRed(),
				defaultBackgroundColorDescriptor.getGreen(), defaultBackgroundColorDescriptor.getBlue()));

		ShellPlugin.getDefault().getPreferenceStore().firePropertyChangeEvent(
				ShellID.SHELL_STYLE_STRING_KEY, null, null);
		super.performDefaults();
	}
}
