/*
 * ShellView.java
 * 
 * Copyright 2003-2004 Stefan Reichert.
 * All Rights Reserved.
 * 
 * This software is the proprietary information of Stefan Reichert.
 * Use is subject to license terms.
 * 
 */
package net.sf.wickedshell.ui.shell;

import java.io.IOException;

import net.sf.wickedshell.ShellID;
import net.sf.wickedshell.action.shell.OpenShellErrorDisplayAction;
import net.sf.wickedshell.facade.IShellFacade;
import net.sf.wickedshell.listener.ShellAdapter;
import net.sf.wickedshell.ui.shell.viewer.IShellViewer;
import net.sf.wickedshell.ui.shell.viewer.ShellViewerKeyAdapter;
import net.sf.wickedshell.ui.shell.viewer.TextShellViewer;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

public class ShellView extends ViewPart {

	/** The <code>IShellFacade</code>. */
	private IShellFacade shellFacade;

	/** The <code>IShellViewer</code>. */
	private IShellViewer shellViewer;

	/** The <code>ShellErrorDisplay</code>. */
	private ShellErrorDisplay errorDisplay;

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new FillLayout());

		// shellViewer = new StyledTextShellViewer(container, SWT.V_SCROLL | SWT.MULTI | SWT.BORDER
		// | SWT.H_SCROLL);
		shellViewer = new TextShellViewer(container, SWT.V_SCROLL | SWT.MULTI | SWT.BORDER
				| SWT.H_SCROLL);

		errorDisplay = new ShellErrorDisplay(getSite().getShell());

		initShellFacade();
		initToolBar();
		initMenuBar();
	}

	private void initShellFacade() {
		try {
			// Create the shell facade
			shellFacade = IShellFacade.Factory.createInstance();
			shellViewer.initialize(shellFacade);
			ShellViewerKeyAdapter.adapt(shellViewer);
			// Register DiposeListener for deactivation
			shellViewer.getShellDisplayControl().addDisposeListener(new DisposeListener() {
				public void widgetDisposed(DisposeEvent event) {
					try {
						shellFacade.deactivate();
					}
					catch (IOException exception) {
						exception.printStackTrace();
					}
				}
			});
			shellFacade.addShellOutputListener(errorDisplay);
			shellFacade.addShellListener(new ShellAdapter() {

				/**
				 * @see net.sf.wickedshell.listener.ShellAdapter#handleActivation()
				 */
				public void handleActivation() {
					ShellView.this.setContentDescription(createContentDescription());
				}
			});
			// Activate shell facade
			shellFacade.activate();
		}
		catch (Exception exception) {
			StringBuffer errorBuffer = new StringBuffer();
			errorBuffer.append("Exception occured: ");
			errorBuffer.append(exception.getClass().getName());
			errorBuffer.append(" --- ");
			errorBuffer.append(exception.getMessage());
			errorBuffer.append(Text.DELIMITER);
			errorBuffer.append("Please refer to error log for further details. ");
			errorBuffer.append("The setup of Wicked Shell failed.");
			Text shellDisplayControl = (Text) shellViewer.getShellDisplayControl();
			shellDisplayControl.append(errorBuffer.toString());
			shellDisplayControl.setEditable(false);
			errorDisplay.handleShellOutput(errorBuffer.toString());
			exception.printStackTrace();
		}
	}

	/**
	 * Creates the content description of this view by inpecting the <code>IShellFacade</code>.
	 * 
	 * @return the content description
	 */
	private String createContentDescription() {
		StringBuffer contentDescriptionBuffer = new StringBuffer();
		contentDescriptionBuffer.append("Wicked Shell ");
		contentDescriptionBuffer.append(ShellID.VERSION);
		contentDescriptionBuffer.append(" - ");
		if (shellFacade.getShellDescriptor() != null) {
			contentDescriptionBuffer.append(shellFacade.getShellDescriptor().getName());
			contentDescriptionBuffer.append(" [Character encoding ");
			contentDescriptionBuffer.append(shellFacade.getShellDescriptor().getCharacterEncoding());
			contentDescriptionBuffer.append("]");
		}
		else {
			contentDescriptionBuffer.append("No Shell available");
		}
		return contentDescriptionBuffer.toString();
	}

	private void initMenuBar() {
		IMenuManager manager = getViewSite().getActionBars().getMenuManager();
		manager.add(new Separator(ShellID.SHELL_VIEW_MENU_GROUP_MANAGE));
		manager.add(new Separator(ShellID.SHELL_VIEW_MENU_GROUP_EXPORT));
		manager.add(new Separator(ShellID.SHELL_VIEW_MENU_GROUP_GENERAL));
		manager.add(new OpenShellErrorDisplayAction(errorDisplay));
		getViewSite().getActionBars().updateActionBars();
	}

	private void initToolBar() {
		IToolBarManager manager = getViewSite().getActionBars().getToolBarManager();
		manager.add(new Separator(ShellID.SHELL_VIEW_MENU_GROUP_MANAGE));
		manager.add(new Separator(ShellID.SHELL_VIEW_MENU_GROUP_GENERAL));
		getViewSite().getActionBars().updateActionBars();
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		shellViewer.getShellDisplayControl().forceFocus();
	}

	/**
	 * @see org.eclipse.ui.IViewPart#init(org.eclipse.ui.IViewSite)
	 */
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
		if (getViewSite().getSecondaryId() != null) {
			StringBuffer partnameBuffer = new StringBuffer();
			partnameBuffer.append(getPartName());
			partnameBuffer.append(" (additional)");
			super.setPartName(partnameBuffer.toString());
		}

	}

	public IShellViewer getShellViewer() {
		return shellViewer;
	}

}
