/*
 * TextShellViewer.java
 * 
 * Copyright 2005-2006 Stefan Reichert.
 * All Rights Reserved.
 * 
 * This software is the proprietary information of Stefan Reichert.
 * Use is subject to license terms.
 * 
 */
package net.sf.wickedshell.ui.shell.viewer;

import net.sf.wickedshell.ShellID;
import net.sf.wickedshell.ShellPlugin;
import net.sf.wickedshell.domain.DomainPlugin;
import net.sf.wickedshell.domain.completion.ICompletion.Completion;
import net.sf.wickedshell.domain.style.IColorDescriptor;
import net.sf.wickedshell.domain.style.IFontDescriptor;
import net.sf.wickedshell.facade.IShellFacade;
import net.sf.wickedshell.listener.IShellListener;
import net.sf.wickedshell.ui.shell.viewer.proposal.ShellContentProposalProvider;
import net.sf.wickedshell.util.ResourceManager;

import org.eclipse.jface.bindings.keys.KeyStroke;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.IContentProposal;
import org.eclipse.jface.fieldassist.IContentProposalListener;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;

/**
 * @author Stefan Reichert
 * @since 18.10.2006
 */
public class TextShellViewer implements IShellViewer, IShellListener {

	/** The <code>Text</code> functioning as display. */
	private Text shellDisplayControl;

	/** The active <code>IShellDescriptor</code>. */
	private IShellFacade shellFacade;

	/** Flag whether scrolllock is enabled. */
	private boolean scrollLock;

	/** The adapter for the proposal. */
	private ContentProposalAdapter adapter;

	/** The last line of the shell. */
	private String lastLine;

	/**
	 * Constructor for TextShellViewer.
	 * 
	 * @param shellDisplayControl
	 *        The <code>Text</code> functioning as display
	 */
	public TextShellViewer(Composite parent, int style) {
		this(new Text(parent, style));
	}

	/**
	 * Constructor for TextShellViewer.
	 * 
	 * @param shellDisplayControl
	 *        The <code>Text</code> functioning as display
	 */
	public TextShellViewer(final Text shellDisplayControl) {
		super();
		this.shellDisplayControl = shellDisplayControl;
		refreshFont();
		refreshForeground();
		refreshBackground();
		initFieldAssist();
		this.lastLine = new String();
		this.shellDisplayControl.addModifyListener(new ModifyListener() {
			/**
			 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
			 */
			public void modifyText(ModifyEvent event) {
				int lastLineStart = shellDisplayControl.getText().lastIndexOf(
						shellDisplayControl.getLineDelimiter());
				if (lastLineStart == -1) {
					lastLine = shellDisplayControl.getText();
				}
				else {
					lastLine = shellDisplayControl.getText().substring(
							lastLineStart + shellDisplayControl.getLineDelimiter().length());
				}
			}
		});
		ShellPlugin.getDefault().getPreferenceStore().addPropertyChangeListener(
				new org.eclipse.jface.util.IPropertyChangeListener() {
					/**
					 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
					 */
					public void propertyChange(PropertyChangeEvent event) {
						if (!TextShellViewer.this.shellDisplayControl.isDisposed()) {
							String property = event.getProperty();
							if (property.equals(ShellID.SHELL_STYLE_STRING_KEY)) {
								refreshBackground();
								refreshForeground();
								refreshFont();
							}
						}
					}
				});
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#appendToCurrentCommand(java.lang.String)
	 */
	public void appendToCurrentCommand(String text) {
		shellDisplayControl.append(text);
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getCurrentCaretPosition()
	 */
	public int getCurrentCaretPosition() {
		return shellDisplayControl.getCaretPosition();
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getCurrentCommand()
	 */
	public String getCurrentCommand() {
		return shellDisplayControl.getText().substring(getCurrentCommandOffset());
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getCurrentCommandOffset()
	 */
	public int getCurrentCommandOffset() {
		if (lastLine.indexOf(shellFacade.getShellDescriptor().getPathDelimiter()) != -1) {
			int delimiterLength = shellFacade.getShellDescriptor().getPathDelimiter().length();
			return shellDisplayControl.getText().length()
					- (lastLine.length() - getCurrentPath().length() - delimiterLength);
		}
		else {
			return shellDisplayControl.getText().length() + 1;
		}
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getCurrentPath()
	 */
	public String getCurrentPath() {
		int pathEnd = lastLine.indexOf(shellFacade.getShellDescriptor().getPathDelimiter());
		return lastLine.substring(0, pathEnd);
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getCurrentTextSelection()
	 */
	public Point getCurrentTextSelection() {
		return shellDisplayControl.getSelection();
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getCurrentTextSelectionCount()
	 */
	public int getCurrentTextSelectionCount() {
		return shellDisplayControl.getSelectionCount();
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#isCurrentCaretPositionValid()
	 */
	public boolean isCurrentCaretPositionValid() {
		return shellDisplayControl.getSelection().x >= getCurrentCommandOffset();
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#setCurrentCaretPosition(int)
	 */
	public void setCurrentCaretPosition(int caretPosition) {
		shellDisplayControl.setSelection(caretPosition);
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#setCurrentCommand(java.lang.String)
	 */
	public void setCurrentCommand(String command) {
		shellDisplayControl.setSelection(getCurrentCommandOffset(), shellDisplayControl.getText()
				.length());
		shellDisplayControl.insert(command);
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#setCurrentTextSelection(org.eclipse.swt.graphics.Point)
	 */
	public void setCurrentTextSelection(Point selection) {
		shellDisplayControl.setSelection(selection);
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#clear()
	 */
	public void clear() {
		String currentPath = getCurrentPath();
		shellDisplayControl.setText("");
		shellDisplayControl.append(currentPath);
		shellDisplayControl.append(shellFacade.getShellDescriptor().getPathDelimiter());
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#reset()
	 */
	public void reset() {
		shellDisplayControl.setText("");
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getContentLength()
	 */
	public int getContentLength() {
		return shellDisplayControl.getText().length();
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#isPathDelimiterAvailableInLastLine()
	 */
	public boolean isPathDelimiterAvailableInLastLine() {
		return lastLine.indexOf(shellFacade.getShellDescriptor().getPathDelimiter()) == -1;
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#refreshBackground()
	 */
	public void refreshBackground() {
		IColorDescriptor backgroundColorDescriptor = DomainPlugin.getDefault().getBackgroundColor();
		RGB backgroundRGB = new RGB(backgroundColorDescriptor.getRed(), backgroundColorDescriptor
				.getGreen(), backgroundColorDescriptor.getBlue());
		shellDisplayControl.setBackground(ResourceManager.getColor(backgroundRGB));
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#refreshFont()
	 */
	public void refreshFont() {
		IFontDescriptor fontDescriptor;
		fontDescriptor = DomainPlugin.getDefault().getFontDescriptor();
		shellDisplayControl.setFont(ResourceManager.getFont(fontDescriptor.getName(), fontDescriptor
				.getHeight(), fontDescriptor.getStyle()));
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#refreshForeground()
	 */
	public void refreshForeground() {
		IColorDescriptor foregroundColorDescriptor = DomainPlugin.getDefault().getForegroundColor();
		RGB foregroundRGB = new RGB(foregroundColorDescriptor.getRed(), foregroundColorDescriptor
				.getGreen(), foregroundColorDescriptor.getBlue());
		shellDisplayControl.setForeground(ResourceManager.getColor(foregroundRGB));
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#initialize(net.sf.wickedshell.facade.IShellFacade)
	 */
	public void initialize(IShellFacade shellFacade) {
		shellDisplayControl.setEditable(true);
		this.shellFacade = shellFacade;
		this.shellFacade.addShellOutputListener(this);
		this.shellFacade.addShellListener(this);
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellOutputListener#handleShellErrorOutput(java.lang.String)
	 */
	public void handleShellErrorOutput(final String errorOutput) {
		if (!shellDisplayControl.isDisposed()) {
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					if (!shellDisplayControl.isDisposed()) {
						int scrollIndex = shellDisplayControl.getTopIndex();
						shellDisplayControl.append(errorOutput);
						if (scrollLock) {
							shellDisplayControl.setTopIndex(scrollIndex);
						}
					}
				}
			});
		}
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellOutputListener#handleShellOutput(java.lang.String)
	 */
	public void handleShellOutput(final String output) {
		if (!shellDisplayControl.isDisposed()) {
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					if (!shellDisplayControl.isDisposed()) {
						int scrollIndex = shellDisplayControl.getTopIndex();
						shellDisplayControl.append(output);
						if (scrollLock) {
							shellDisplayControl.setTopIndex(scrollIndex);
						}
					}
				}
			});
		}
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getShellDisplayControl()
	 */
	public Control getShellDisplayControl() {
		return shellDisplayControl;
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#setScrollLock(boolean)
	 */
	public void setScrollLock(boolean scrollLock) {
		this.scrollLock = scrollLock;
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getShellFacade()
	 */
	public IShellFacade getShellFacade() {
		return shellFacade;
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellListener#handleCommandExecution(java.lang.String)
	 */
	public void handleCommandExecution(String command) {
		if ("cls".equals(command.toLowerCase())) {
			clear();
		}
		else if ("exit".equals(command.toLowerCase())) {
			setCurrentCommand(command);
		}
		else if (shellFacade.getShellDescriptor().isExecutedCommandProvided()) {
			setCurrentCommand(new String());
		}
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellListener#handleRestart()
	 */
	public void handleRestart() {
		reset();

		refreshFont();
		refreshForeground();
		refreshBackground();
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellListener#handleActivation()
	 */
	public void handleActivation() {
		shellDisplayControl.setEditable(true);
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellListener#handleDeactivation()
	 */
	public void handleDeactivation() {
		shellDisplayControl.setEditable(false);
		shellDisplayControl.append(Text.DELIMITER);
		shellDisplayControl.append("--- Shell was terminated; ---");
	}

	/**
	 * Initializes the field assist which provides command completion.
	 */
	private void initFieldAssist() {
		KeyStroke keyStroke = KeyStroke.getInstance(SWT.CTRL, ' ');
		adapter = new ContentProposalAdapter(shellDisplayControl, new TextContentAdapter(),
				new ShellContentProposalProvider(this), keyStroke, new char[] { SWT.TAB });
		adapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_IGNORE);
		adapter.addContentProposalListener(new IContentProposalListener() {
			/**
			 * @see org.eclipse.jface.fieldassist.IContentProposalListener#proposalAccepted(org.eclipse.jface.fieldassist.IContentProposal)
			 */
			public void proposalAccepted(IContentProposal proposal) {
				setCurrentCommand(proposal.getContent());
			}
		});
		adapter.setLabelProvider(new LabelProvider() {

			/**
			 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
			 */
			public String getText(Object element) {
				Completion completion = (Completion) element;
				return completion.getLabel();
			}

			/**
			 * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
			 */
			public Image getImage(Object element) {
				Completion completion = (Completion) element;
				return ResourceManager.getPluginImage(ShellPlugin.getDefault(), completion
						.getImagePath());
			}
		});
	}

	/**
	 * @see net.sf.wickedshell.ui.shell.viewer.IShellViewer#getViewerContent()
	 */
	public String getViewerContent() {
		return shellDisplayControl.getText();
	}
}
